## $Id: interfaces.py 12888 2015-04-26 19:45:46Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

from zope import schema
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, academic_sessions_vocab, validate_email)
from waeup.kofa.schema import FormattedDate, TextLineChoice
from waeup.kofa.students.vocabularies import nats_vocab, GenderSource
from waeup.kofa.applicants.interfaces import contextual_reg_num_source
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit, INigeriaPGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    )
from kofacustom.wdu.interfaces import MessageFactory as _
from kofacustom.wdu.payments.interfaces import ICustomOnlinePayment

class ICustomUGApplicant(IApplicantBaseData):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the UG_OMIT* tuples.
    """
    sex = schema.Choice(
        title = _(u'Sex'),
        source = GenderSource(),
        required = False,
        )
    marit_stat = schema.Choice(
        title = u'Maritual Status',
        required = False,
        vocabulary = SimpleKofaVocabulary(
            (_('Single'), 'single'),
            (_('Married'), 'married'),
            (_('Widowed'), 'widowed'),
            (_('Divorced'), 'divorced'),
            (_('Separated'), 'separated'),)
        )
    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    religion = schema.Choice(
        title = u'Religion',
        default = 'no_say',
        required = False,
        vocabulary = SimpleKofaVocabulary(
            (_('Muslim'), 'muslim'),
            (_('Christian'), 'christian'),
            (_('Others'), 'others'),
            (_('Prefer not to say'), 'no_say'),)
        )
    #perm_address = schema.Text(
    #    title = _(u'Permanent Address'),
    #    required = False,
    #    )
    next_kin_address = schema.Text(
        title = _(u'Next of Kin Address'),
        required = False,
        )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    olevel_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    olevel_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    olevel_exam_number = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )
    olevel_exam_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    olevel_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        default = [],
        )
    olevel_type2 = schema.Choice(
        title = _(u'2nd Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    olevel_school2 = schema.TextLine(
        title = _(u'2nd Institution Attended'),
        required = False,
        readonly = False,
        )
    olevel_exam_number2 = schema.TextLine(
        title = _(u'2nd Exam Number'),
        required = False,
        readonly = False,
        )
    olevel_exam_date2 = FormattedDate(
        title = _(u'2nd Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    olevel_results2 = schema.List(
        title = _(u'2nd Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        default = [],
        )
    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    employer = schema.TextLine(
        title = _(u'Employer'),
        required = False,
        readonly = False,
        )
    emp_position = schema.TextLine(
        title = _(u'Employer Position'),
        required = False,
        readonly = False,
        )
    emp_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    employer2 = schema.TextLine(
        title = _(u'2nd Employer'),
        required = False,
        readonly = False,
        )
    emp2_position = schema.TextLine(
        title = _(u'2nd Employer Position'),
        required = False,
        readonly = False,
        )
    emp2_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp2_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    body_responsible_address = schema.Text(
        title = _(u'Name and Adress of Body'),
        description = _(
         'Name and address of body who will be responsible for the fees. '
         'If admission to the university is offered '
         'a guarantor\'s form that all the fees will be paid will be sworn to '
         'by the person or body named. The student will not be allowed into the '
         'university until the guarantor\'s form is signed and returned '
         'to the institution.'),
        required = False,
       )
    jamb_subjects = schema.Text(
        title = _(u'Subjects and Scores'),
        description = _(u'(one subject with score per line)'),
        required = False,
        )
    jamb_score = schema.Int(
        title = _(u'Total JAMB Score'),
        required = False,
        )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = False,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Int(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    aggregate = schema.Int(
        title = _(u'Aggregate Score (%)'),
        description = _(u'(average of relative JAMB and PUTME scores)'),
        required = False,
        )
    result_uploaded = schema.Bool(
        title = _(u'Result uploaded'),
        default = False,
        required = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

ICustomUGApplicant[
    'sex'].order =  IApplicantBaseData['sex'].order

class ICustomPGApplicant(INigeriaPGApplicant):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """


class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant):
    """An interface for both types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicant data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomPGApplicantEdit(INigeriaPGApplicantEdit):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class ICustomPUTMEApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantUpdateByRegNo(INigeriaApplicantUpdateByRegNo):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """
